<?php
// wbb imports
require_once(WBB_DIR.'lib/data/page/seo/WBBSEORewriter.class.php');

// wcf imports
require_once(WCF_DIR.'lib/data/page/seo/SEOUtil.class.php');

/**
 * Redirects (HTTP 301) to SEO links.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2009 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb.seo
 * @subpackage	data.page.seo
 * @category 	Burning Board
 */
class WBBSEORedirecter extends WBBSEORewriter {
	public $encodeHTML = false;
	public $className = '';
	public $requestURI = '';
	public $object = null;
	
	/**
	 * Redirects (HTTP 301) to SEO links.
	 *
	 * @param	string		$requestURI
	 * @param	object		$object
	 */
	public function redirect($requestURI, $object) {
		$this->requestURI = $requestURI;
		$this->object = $object;
		$this->className = get_class($object);
		if (SEO_ENABLE_301_REDIRECTS && WCF::getSession()->requestMethod == 'GET') {
			switch ($this->className) {
				case 'IndexPage':
					if (SEO_REWRITE_INDEX) {
						if (preg_match('~^index\.php(\?s=[a-f0-9]{40})?$~', $requestURI, $match) || preg_match('~^index\.php\?page=Index$~', $requestURI)) {
							$this->forward($this->parseIndexURLs((isset($match[1]) ? $match[1] : ''), SEO_REWRITE_INDEX_FORMAT));
						}
						$format = $this->parseIndexURLs(SID_ARG_1ST);
						if (preg_replace('~\?.*~', '', $requestURI) != preg_replace('~\?.*~', '', $format)) {
							$this->forward($format);
						}
					}
					break;
				case 'BoardPage':
					if (SEO_REWRITE_BOARD) {
						if (preg_match('~^index\.php\?page=Board&boardID=(\d+)&pageNo=(\d+)(.*)?$~', $requestURI, $match)) {
							$url = $this->replaceMultipleBoardURLsCallback($match);
							if ($url != $match[0]) $this->forward($url);
						}
						else if (preg_match('~^index\.php\?page=Board&boardID=(\d+)(.*)?$~', $requestURI, $match)) {
							$url = $this->replaceBoardURLsCallback($match);
							if ($url != $match[0]) $this->forward($url);
						}
						
						if ($object->pageNo > 1) {
							$format = $this->parseMultipleBoardURLs($object->boardID, $object->pageNo, SID_ARG_1ST);
						}
						else {
							$format = $this->parseBoardURLs($object->boardID, SID_ARG_1ST);
						}
						if (preg_replace('~\?.*~', '', $requestURI) != preg_replace('~\?.*~', '', $format)) {
							$this->forward($format);
						}
					}
					break;
				case 'ThreadPage':
					if (SEO_REWRITE_THREAD) {
						if (preg_match('~^index\.php\?page=Thread&threadID=(\d+)&action=firstNew(.*)?$~', $requestURI, $match)) {
							$this->cacheThreads(array($match[1]));
							$url = $this->replaceFirstNewPostURLsCallback($match);
							if ($url != $match[0]) $this->forward($url);
						}
						else if (preg_match('~^index\.php\?page=Thread&threadID=(\d+)&action=lastPost(.*)?$~', $requestURI, $match)) {
							$this->cacheThreads(array($match[1]));
							$url = $this->replaceLastPostURLsCallback($match);
							if ($url != $match[0]) $this->forward($url);
						}
						else if (preg_match('~^index\.php\?page=Thread&threadID=(\d+)&pageNo=(\d+)(.*)?$~', $requestURI, $match)) {
							$this->cacheThreads(array($match[1]));
							$url = $this->replaceMultipleThreadURLsCallback($match);
							if ($url != $match[0]) $this->forward($url);
						}
						else if (preg_match('~^index\.php\?page=Thread&threadID=(\d+)(.*)?$~', $requestURI, $match)) {
							$this->cacheThreads(array($match[1]));
							$url = $this->replaceThreadURLsCallback($match);
							if ($url != $match[0]) $this->forward($url);
						}
						else if (preg_match('~^index\.php\?page=Thread&postID=(\d+)(.*)?$~', $requestURI, $match)) {
							$this->cachePosts(array($match[1]));
							$url = $this->replacePostURLsCallback($match);
							if ($url != $match[0]) $this->forward($url);
						}
						
						$this->addCachedThread($object->thread);
						if ($object->postID) {
							if ($object->pageNo == 1) {
								$url = $this->parseThreadURLs($object->threadID, SID_ARG_1ST.'#post'.$object->postID);
							}
							else {
								$url = $this->parseMultipleThreadURLs($object->threadID, $object->pageNo, SID_ARG_1ST.'#post'.$object->postID);
							}
							$this->forward($url);
						}
						
						if ($object->pageNo > 1) {
							$format = $this->parseMultipleThreadURLs($object->threadID, $object->pageNo, SID_ARG_1ST);
						}
						else {
							$format = $this->parseThreadURLs($object->threadID, SID_ARG_1ST);
						}
						if (preg_replace('~\?.*~', '', $requestURI) != preg_replace('~\?.*~', '', $format)) {
							$this->forward($format);
						}
					}
					break;
				case 'TaggedObjectsPage':
					if (SEO_REWRITE_TAG) {
						if (preg_match('~^index\.php\?page=TaggedObjects&tagID=(\d+)(.*)?$~', $requestURI, $match)) {
							$this->cacheTags(array($match[1]));
							$url = $this->replaceTagURLsCallback($match);
							if ($url != $match[0]) $this->forward($url);
						}
						
						if ($object->tagObj !== null) {
							$this->addCachedTag($object->tagObj);
							$format = $this->parseTagURLs($object->tagObj->getID(), SID_ARG_1ST);
							if (preg_replace('~\?.*~', '', $requestURI) != preg_replace('~\?.*~', '', $format)) {
								$this->forward($format);
							}
						}
					}
					break;
				case 'UserPage':
					if (SEO_REWRITE_USER_PROFILE) {
						if (preg_match('~^index\.php\?page=User&userID=(\d+)(.*)?$~', $requestURI, $match)) {
							$this->cacheUsers(array($match[1]));
							$url = $this->replaceUserProfileURLsCallback($match);
							if ($url != $match[0]) $this->forward($url);
						}
						
						$this->addCachedUser($object->frame->getUser());
						$format = $this->parseUserProfileURLs($object->frame->getUserID(), SID_ARG_1ST);
						if (preg_replace('~\?.*~', '', $requestURI) != preg_replace('~\?.*~', '', $format)) {
							$this->forward($format);
						}
					}
					break;
				default:
					// call event
					EventHandler::fireAction($this, 'redirect');
			}
		}
		
		if (SEO_CUSTOM_REDIRECTS) {
			$redirects = explode("\n", StringUtil::unifyNewlines(SEO_CUSTOM_REDIRECTS));
			foreach ($redirects as $redirect) {
				if (preg_match("~'([^']+)'='([^']+)'~", $redirect, $match)) {
					if (preg_match('~'.$match[1].'~', $requestURI)) {
						$this->forward(preg_replace('~'.$match[1].'~', $match[2], $requestURI));
					}
				}
			}
		}
	}
	
	/**
	 * Does the redirect.
	 *
	 * @param	string		$url
	 */
	public function forward($url) {
		HeaderUtil::redirect($url, true, true);
		exit;
	}
	
	/**
	 * @see	SEORewriter::appendQueryString()
	 */
	public function appendQueryString($link, $queryString) {
		return SEORewriter::appendQueryString($link, $queryString);
	}
}
?>