<?php
// wbb imports
require_once(WBB_DIR.'lib/data/board/Board.class.php');
require_once(WBB_DIR.'lib/data/thread/Thread.class.php');
require_once(WBB_DIR.'lib/data/post/Post.class.php');

// wcf imports
require_once(WCF_DIR.'lib/data/page/seo/SEORewriter.class.php');
require_once(WCF_DIR.'lib/data/page/seo/SEOUtil.class.php');
require_once(WCF_DIR.'lib/data/tag/Tag.class.php');
require_once(WCF_DIR.'lib/system/event/EventHandler.class.php');

/**
 * Rewrite SEO links in HTML code.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2009 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb.seo
 * @subpackage	data.page.seo
 * @category 	Burning Board
 */
class WBBSEORewriter extends SEORewriter {
	/**
	 * cached page title
	 * 
	 * @var	string
	 */
	protected $cachedPageTitle = null;

	/**
	 * cached threads
	 * 
	 * @var array	threadID => array(topic, board id)
	 */
	protected $cachedThreads = array();
	
	/**
	 * cached posts
	 * 
	 * @var	array	postID => array(thread topic, board id)
	 */
	protected $cachedPosts = array();
	
	/**
	 * cached board titles
	 * 
	 * @var	array	boardID => board title
	 */
	protected $cachedBoardTitles = array();
	
	/**
	 * cached boards
	 * 
	 * @var	array	boardID => array(parent board titles)
	 */
	protected $cachedParentBoards = array();
	
	/**
	 * cached tags
	 * 
	 * @var	array
	 */
	protected $cachedTags = array();
	
	/**
	 * cached users
	 * 
	 * @var	array
	 */
	protected $cachedUsers = array();
	
	/**
	 * page content
	 * 
	 * @var	text
	 */
	public $text = '';
	
	/**
	 * Rewrite SEO links in HTML code.
	 * 
	 * @param	string		$text
	 * @return	string
	 */
	public function rewrite($text) {
		$this->text = $text;
		
		// call event
		EventHandler::fireAction($this, 'rewriteStart');
		
		if (SEO_REWRITE_INDEX) {
			$this->text = preg_replace_callback('~(?<=href=")index\.php\?page=Index([^"]+)?(?=")~', array($this, 'replaceIndexURLsCallback'), $this->text);
		}
		
		if (SEO_REWRITE_BOARD) {
			$this->text = preg_replace_callback('~(?<=href=")index\.php\?page=Board&amp;boardID=(\d+)&amp;pageNo=(\d+)([^"]+)?(?=")~', array($this, 'replaceMultipleBoardURLsCallback'), $this->text);
			$this->text = preg_replace_callback('~(?<=href=")index\.php\?page=Board&amp;boardID=(\d+)([^"]+)?(?=")~', array($this, 'replaceBoardURLsCallback'), $this->text);
		}
		
		if (SEO_REWRITE_THREAD) {
			// cache thread ids
			if (preg_match_all('~(?<=href=")index\.php\?page=Thread&amp;threadID=(\d+)(?:[^"]+)?(?=")~', $this->text, $matches)) {
				$this->cacheThreads($matches[1]);
			}
			
			// cache post ids
			if (preg_match_all('~(?<=href=")index\.php\?page=Thread&amp;postID=(\d+)(?:[^"]+)?(?=")~', $this->text, $matches)) {
				$this->cachePosts($matches[1]);
			}
			
			if (count($this->cachedThreads)) {
				$this->text = preg_replace_callback('~(?<=href=")index\.php\?page=Thread&amp;threadID=(\d+)&amp;action=firstNew([^"]+)?(?=")~', array($this, 'replaceFirstNewPostURLsCallback'), $this->text);
				$this->text = preg_replace_callback('~(?<=href=")index\.php\?page=Thread&amp;threadID=(\d+)&amp;action=lastPost([^"]+)?(?=")~', array($this, 'replaceLastPostURLsCallback'), $this->text);
				$this->text = preg_replace_callback('~(?<=href=")index\.php\?page=Thread&amp;threadID=(\d+)&amp;pageNo=(\d+)([^"]+)?(?=")~', array($this, 'replaceMultipleThreadURLsCallback'), $this->text);
				$this->text = preg_replace_callback('~(?<=href=")index\.php\?page=Thread&amp;threadID=(\d+)([^"]+)?(?=")~', array($this, 'replaceThreadURLsCallback'), $this->text);
			}
			
			if (count($this->cachedPosts)) {
				$this->text = preg_replace_callback('~(?<=href=")index\.php\?page=Thread&amp;postID=(\d+)([^"]+)?(?=")~', array($this, 'replacePostURLsCallback'), $this->text);
			}
		}
		
		if (SEO_REWRITE_TAG) {
			// cache tag ids
			if (preg_match_all('~(?<=href=")index\.php\?page=TaggedObjects&amp;tagID=(\d+)(?:[^"]+)?(?=")~', $this->text, $matches)) {
				$this->cacheTags($matches[1]);
			}
			
			if (count($this->cachedTags)) {
				$this->text = preg_replace_callback('~(?<=href=")index\.php\?page=TaggedObjects&amp;tagID=(\d+)([^"]+)?(?=")~', array($this, 'replaceTagURLsCallback'), $this->text);
			}
		}
		
		if (SEO_REWRITE_USER_PROFILE) {
			// cache user ids
			if (preg_match_all('~(?<=href=")index\.php\?page=User&amp;userID=(\d+)(?:[^"]+)?(?=")~', $this->text, $matches)) {
				$this->cacheUsers($matches[1]);
			}
			
			if (count($this->cachedUsers)) {
				$this->text = preg_replace_callback('~(?<=href=")index\.php\?page=User&amp;userID=(\d+)([^"]+)?(?=")~', array($this, 'replaceUserProfileURLsCallback'), $this->text);
			}
		}
		
		// call event
		EventHandler::fireAction($this, 'rewriteEnd');
		
		$this->text = parent::rewrite($this->text);
		
		return $this->text;
	}
	
	/**
	 * Parses index urls.
	 * 
	 * @param	string		$queryString
	 * @param	string		$string
	 * 
	 * @return	string
	 */
	public function parseIndexURLs($queryString, $string = SEO_REWRITE_INDEX_FORMAT) {
		if ($this->cachedPageTitle === null) {
			$this->cachedPageTitle = SEOUtil::formatString(PAGE_TITLE);
			if ($this->encodeHTML) $this->cachedPageTitle = StringUtil::encodeHTML($this->cachedPageTitle);
		}

		/* parse string */
		$string = str_replace('{PAGE_TITLE}',  $this->cachedPageTitle, $string);
		
		// append query string
		$string = $this->appendQueryString($string, $queryString);
		
		return $string;
	}
	
	/**
	 * Parses multiple board urls.
	 * 
	 * @param	integer		$boardID
	 * @param	integer		$pageNo
	 * @param	string		$queryString
	 * @param	string		$string
	 * 
	 * @return	string
	 */
	public function parseMultipleBoardURLs($boardID, $pageNo, $queryString, $string = SEO_REWRITE_BOARD_MULTIPLE_FORMAT) {
		if ($pageNo == 1) $string = SEO_REWRITE_BOARD_FORMAT;
		else $string = str_replace('{BOARD_PAGE_NO}', $pageNo, $string);

		return $this->parseBoardURLs($boardID, $queryString, $string);
	}
	
	/**
	 * Parses board urls.
	 * 
	 * @param	integer		$boardID
	 * @param	string		$queryString
	 * @param	string		$string
	 * 
	 * @return	string
	 */
	public function parseBoardURLs($boardID, $queryString, $string = SEO_REWRITE_BOARD_FORMAT) {
		/* parse string */
		// board id
		$string = str_replace('{BOARD_ID}', $boardID, $string);
		
		// board title
		if (strpos($string, '{BOARD_TITLE}') !== false) {
			$title = $this->getBoardTitle($boardID);
			$string = str_replace('{BOARD_TITLE}', $title, $string);
		}
		
		// parent boards
		$string = $this->parseParentBoards($string, $boardID);
		
		// append query string
		$string = $this->appendQueryString($string, $queryString);
		
		return $string;
	}
	
	/**
	 * Parses multiple thread urls.
	 * 
	 * @param	integer		$threadID
	 * @param	integer		$pageNo
	 * @param	string		$queryString
	 * @param	string		$string
	 * 
	 * @return	string
	 */
	public function parseMultipleThreadURLs($threadID, $pageNo, $queryString, $string = SEO_REWRITE_THREAD_MULTIPLE_FORMAT) {
		if ($pageNo == 1) $string = SEO_REWRITE_THREAD_FORMAT;
		else $string = str_replace('{THREAD_PAGE_NO}', $pageNo, $string);

		return $this->parseThreadURLs($threadID, $queryString, $string);
	}
	
	/**
	 * Parses thread urls.
	 * 
	 * @param	integer		$threadID
	 * @param	string		$queryString
	 * @param	string		$string
	 * 
	 * @return	string
	 */
	public function parseThreadURLs($threadID, $queryString, $string = SEO_REWRITE_THREAD_FORMAT) {
		if (!isset($this->cachedThreads[$threadID])) return false;
		$thread = $this->cachedThreads[$threadID];
		
		/* parse string */
		// thread id
		$string = str_replace('{THREAD_ID}', $threadID, $string);
		
		// board id
		$string = str_replace('{BOARD_ID}', $thread['boardID'], $string);
		
		// board title
		if (strpos($string, '{BOARD_TITLE}') !== false) {
			$title = $this->getBoardTitle($thread['boardID']);
			$string = str_replace('{BOARD_TITLE}', $title, $string);
		}
		
		// thread topic
		$string = str_replace('{THREAD_TOPIC}', $thread['topic'], $string);
		
		// parent boards
		$string = $this->parseParentBoards($string, $thread['boardID']);
		
		// append query string
		$string = $this->appendQueryString($string, $queryString);
		
		return $string;
	}
	
	/**
	 * Parses post urls.
	 * 
	 * @param	integer		$postID
	 * @param	string		$queryString
	 * @param	string		$string
	 * 
	 * @return	string
	 */
	public function parsePostURLs($postID, $queryString, $string = SEO_REWRITE_THREAD_POST_FORMAT) {
		if (!isset($this->cachedPosts[$postID])) return false;
		$post = $this->cachedPosts[$postID];
		
		/* parse string */
		// post id
		$string = str_replace('{POST_ID}', $postID, $string);
		
		// board id
		$string = str_replace('{BOARD_ID}', $post['boardID'], $string);
		
		// thread id
		$string = str_replace('{THREAD_ID}', $post['threadID'], $string);
		
		// board title
		if (strpos($string, '{BOARD_TITLE}') !== false) {
			$title = $this->getBoardTitle($post['boardID']);
			$string = str_replace('{BOARD_TITLE}', $title, $string);
		}
		
		// thread title
		$string = str_replace('{THREAD_TOPIC}', $post['topic'], $string);
		
		// parent boards
		$string = $this->parseParentBoards($string, $post['boardID']);
		
		// append query string
		$string = $this->appendQueryString($string, $queryString);
		
		return $string;
	}
	
	/**
	 * Parses tag urls.
	 * 
	 * @param	integer		$tagID
	 * @param	string		$queryString
	 * @param	string		$string
	 * 
	 * @return	string
	 */
	public function parseTagURLs($tagID, $queryString, $string = SEO_REWRITE_TAG_FORMAT) {
		if (!isset($this->cachedTags[$tagID])) return false;
		$tag = $this->cachedTags[$tagID];
		
		/* parse string */
		// tag id
		$string = str_replace('{TAG_ID}', $tagID, $string);
		
		// tag name
		$string = str_replace('{TAG_NAME}', $tag['name'], $string);
		
		// append query string
		$string = $this->appendQueryString($string, $queryString);
		
		return $string;
	}
	
	/**
	 * Parses user profile urls.
	 * 
	 * @param	integer		$userID
	 * @param	string		$queryString
	 * @param	string		$string
	 * 
	 * @return	string
	 */
	public function parseUserProfileURLs($userID, $queryString, $string = SEO_REWRITE_USER_PROFILE_FORMAT) {
		if (!isset($this->cachedUsers[$userID])) return false;
		$user = $this->cachedUsers[$userID];
		
		/* parse string */
		// user id
		$string = str_replace('{USER_ID}', $userID, $string);
		
		// username
		$string = str_replace('{USERNAME}', $user['name'], $string);
		
		// append query string
		$string = $this->appendQueryString($string, $queryString);
		
		return $string;
	}
	
	/**
	 * Parses 'parent boards' tag.
	 * 
	 * @param	string		$string
	 * @param	integer		$boardID
	 * 
	 * @return	string
	 */
	public function parseParentBoards($string, $boardID) {
		return preg_replace("/\{PARENT_BOARDS\}/e", '$this->getParentBoards('.$boardID.')', $string);
	}
	
	/**
	 * Clears the cache.
	 */
	public function clearCache() {
		$this->cachedThreads = array();
		$this->cachedBoardTitles = array();
	}
	
	/**
	 * Caches threads.
	 * 
	 * @param	array		$threadIDs
	 */
	public function cacheThreads($threadIDs) {
		$sql = "SELECT	threadID, topic, boardID
			FROM	wbb".WBB_N."_thread
			WHERE	threadID IN (".implode(',', $threadIDs).")";
		$result = WCF::getDB()->sendQuery($sql);
		while ($row = WCF::getDB()->fetchArray($result)) {
			$this->addCachedThread(new Thread(null, $row));
		}
	}
	
	/**
	 * Adds a thread of the cache.
	 * 
	 * @param	Thread		$thread
	 */
	public function addCachedThread(Thread $thread) {
		try {
			// get board
			$board = Board::getBoard($thread->boardID);
			
			// check permission
			$board->checkPermission('canViewBoard', 'canEnterBoard');
			
			// add to cache
			$topic = SEOUtil::formatString($thread->topic);
			if ($this->encodeHTML) $topic = StringUtil::encodeHTML($topic);
			if ($topic == '') $topic = $thread->threadID;
			$this->cachedThreads[$thread->threadID] = array('topic' => $topic, 'boardID' => $thread->boardID);
		}
		catch (Exception $e) {}
	}
	
	/**
	 * Caches posts.
	 * 
	 * @param	array		$threadIDs
	 */
	public function cachePosts($postIDs) {
		$sql = "SELECT		post.postID, thread.topic, thread.boardID, post.threadID
			FROM		wbb".WBB_N."_post post
			LEFT JOIN	wbb".WBB_N."_thread thread
			ON		(thread.threadID = post.threadID)
			WHERE		post.postID IN (".implode(',', $postIDs).")";
		$result = WCF::getDB()->sendQuery($sql);
		while ($row = WCF::getDB()->fetchArray($result)) {
			$this->addCachedPost(new Post(null, $row));
		}
	}
	
	/**
	 * Adds a post of the cache.
	 * 
	 * @param	Post		$post
	 */
	public function addCachedPost(Post $post) {
		try {
			// get board
			$board = Board::getBoard($post->boardID);
			
			// check permissions
			$board->checkPermission('canViewBoard', 'canEnterBoard', 'canReadThread');
			
			// add to cache
			$topic = SEOUtil::formatString($post->topic);
			if ($this->encodeHTML) $topic = StringUtil::encodeHTML($topic);
			if ($topic == '') $topic = $post->postID;
			$this->cachedPosts[$post->postID] = array('topic' => $topic, 'boardID' => $post->boardID, 'threadID' => $post->threadID);
		}
		catch (Exception $e) {}
	}
	
	/**
	 * Caches tags.
	 * 
	 * @param	array		$tagIDArray
	 */
	public function cacheTags($tagIDArray) {
		$sql = "SELECT	tagID, name
			FROM	wcf".WCF_N."_tag
			WHERE	tagID IN (".implode(',', $tagIDArray).")";
		$result = WCF::getDB()->sendQuery($sql);
		while ($row = WCF::getDB()->fetchArray($result)) {
			$this->addCachedTag(new Tag(null, $row));
		}
	}
	
	/**
	 * Adds a tag of the cache.
	 * 
	 * @param	Tag		$tag
	 */
	public function addCachedTag(Tag $tag) {
		try {
			// add to cache
			$name = SEOUtil::formatString($tag->getName());
			if ($this->encodeHTML) $name = StringUtil::encodeHTML($name);
			if ($name == '') $name = $tag->getID();
			$this->cachedTags[$tag->getID()] = array('name' => $name);
		}
		catch (Exception $e) {}
	}
	
	/**
	 * Caches users.
	 * 
	 * @param	array		$userIDArray
	 */
	public function cacheUsers($userIDArray) {
		$sql = "SELECT	userID, username
			FROM	wcf".WCF_N."_user
			WHERE	userID IN (".implode(',', $userIDArray).")";
		$result = WCF::getDB()->sendQuery($sql);
		while ($row = WCF::getDB()->fetchArray($result)) {
			$this->addCachedUser(new User(null, $row));
		}
	}
	
	/**
	 * Adds a user of the cache.
	 * 
	 * @param	User		$user
	 */
	public function addCachedUser(User $user) {
		try {
			// add to cache
			$name = SEOUtil::formatString($user->username);
			if ($this->encodeHTML) $name = StringUtil::encodeHTML($name);
			if ($name == '') $name = $user->userID;
			$this->cachedUsers[$user->userID] = array('name' => $name);
		}
		catch (Exception $e) {}
	}
	
	/**
	 * Gets board title from cache.
	 * 
	 * @param	integer		$boardID
	 * @return	string
	 */
	public function getBoardTitle($boardID) {
		if (!isset($this->cachedBoardTitles[$boardID])) {
			try {
				$board = Board::getBoard($boardID);
			}
			catch (Exception $e) {
				return false;
			}
			$this->cachedBoardTitles[$boardID] = SEOUtil::formatString(WCF::getLanguage()->get($board->title));
			if ($this->encodeHTML && !preg_match('/^[a-z0-9_\-]+(?:\.[a-z0-9_\-]+)+$/i', $board->title)) {
				$this->cachedBoardTitles[$boardID] = StringUtil::encodeHTML($this->cachedBoardTitles[$boardID]);
			}
			if ($this->cachedBoardTitles[$boardID] == '') {
				$this->cachedBoardTitles[$boardID] = $boardID;
			}
		}
		
		return $this->cachedBoardTitles[$boardID];
	}
	
	/**
	 * Gets parent boards from cache.
	 * 
	 * @param	integer		$boardID
	 * @param	string		$format
	 * @return	string
	 */
	public function getParentBoards($boardID, $format = SEO_REWRITE_PARENT_BOARD_FORMAT) {
		if (!isset($this->cachedParentBoards[$boardID])) {
			// cache board boards
			$this->cachedParentBoards[$boardID] = array();
			try {
				$board = Board::getBoard($boardID);
			}
			catch (Exception $e) {
				return false;
			}
			$parentBoards = $board->getParentBoards();
			foreach ($parentBoards as $parentBoard) {
				$i = $parentBoard->boardID;//count($this->cachedParentBoards[$boardID]);
				$this->cachedParentBoards[$boardID][$i] = SEOUtil::formatString(WCF::getLanguage()->get($parentBoard->title));
				if ($this->encodeHTML && !preg_match('/^[a-z0-9_\-]+(?:\.[a-z0-9_\-]+)+$/i', $parentBoard->title)) {
					$this->cachedParentBoards[$boardID][$i] = StringUtil::encodeHTML($this->cachedParentBoards[$boardID][$i]);
				}
				if ($this->cachedParentBoards[$boardID][$i] == '') {
					$this->cachedParentBoards[$boardID][$i] = $parentBoard->boardID;
				}
			}
		}
		
		// build string
		$string = '';
		foreach ($this->cachedParentBoards[$boardID] as $parentBoardID => $parentBoard) {
			$parentBoardString = $format;
			$parentBoardString = str_replace('{BOARD_ID}', $parentBoardID, $parentBoardString);
			$parentBoardString = str_replace('{BOARD_TITLE}', $parentBoard, $parentBoardString);
			$string .= $parentBoardString;
		}
		
		return $string;
	}
	
	/**
	 * @see	SEORewriter::appendQueryString()
	 */
	public function appendQueryString($link, $queryString) {
		$link = parent::appendQueryString($link, $queryString);
		
		// adds rel="nofollow" to duplicate content links
		if (true && !empty($queryString) && !preg_match('~^(&amp;|\?)s=[a-f0-9]{40}$~', $queryString) && strpos($queryString, '#') !== 0) {
			$link .= '" rel="nofollow';
		}
		
		return $link;
	}
	
	protected function replaceIndexURLsCallback($match) {
		return $this->parseIndexURLs((isset($match[1]) ? $match[1] : ''));
	}
	
	protected function replaceBoardURLsCallback($match) {
		try {
			$board = Board::getBoard($match[1]);
			$board->checkPermission();
		}
		catch (Exception $e) {
			return $match[0];
		}
			
		return $this->parseBoardURLs($match[1], (isset($match[2]) ? $match[2] : ''));
	}
	
	protected function replaceMultipleBoardURLsCallback($match) {
		try {
			$board = Board::getBoard($match[1]);
			$board->checkPermission();
		}
		catch (Exception $e) {
			return $match[0];
		}
			
		return $this->parseMultipleBoardURLs($match[1], $match[2], (isset($match[3]) ? $match[3] : ''));
	}
	
	protected function replaceThreadURLsCallback($match, $format = SEO_REWRITE_THREAD_FORMAT) {
		if (!isset($this->cachedThreads[$match[1]])) {
			return $match[0];
		}
		
		return $this->parseThreadURLs($match[1], (isset($match[2]) ? $match[2] : ''), $format);
	}
	
	protected function replaceFirstNewPostURLsCallback($match) {
		return $this->replaceThreadURLsCallback($match, SEO_REWRITE_THREAD_NEWEST_POST_FORMAT);
	}
	
	protected function replaceLastPostURLsCallback($match) {
		return $this->replaceThreadURLsCallback($match, SEO_REWRITE_THREAD_LAST_POST_FORMAT);
	}
	
	protected function replaceMultipleThreadURLsCallback($match) {
		if (!isset($this->cachedThreads[$match[1]])) {
			return $match[0];
		}
		
		return $this->parseMultipleThreadURLs($match[1], $match[2], (isset($match[3]) ? $match[3] : ''));
	}
	
	protected function replacePostURLsCallback($match) {
		if (!isset($this->cachedPosts[$match[1]])) {
			return $match[0];
		}
		
		return $this->parsePostURLs($match[1], (isset($match[2]) ? $match[2] : ''));
	}
	
	protected function replaceTagURLsCallback($match) {
		if (!isset($this->cachedTags[$match[1]])) {
			return $match[0];
		}
		
		return $this->parseTagURLs($match[1], (isset($match[2]) ? $match[2] : ''));
	}
	
	protected function replaceUserProfileURLsCallback($match) {
		if (!isset($this->cachedUsers[$match[1]])) {
			return $match[0];
		}
		
		return $this->parseUserProfileURLs($match[1], (isset($match[2]) ? $match[2] : ''));
	}
}
?>