<?php
// wcf imports
require_once(WCF_DIR.'lib/system/event/EventHandler.class.php');

/**
 * Writes rewrite rules to .htaccess file.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2009 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb.seo
 * @subpackage	data.page.seo
 * @category 	Burning Board
 */
class WBBSEOUtil {
	public static $options = null;
	public static $file = null;
	
	/**
	 * @deprecated use WBBSEOUtil::rebuildConfigFile() instead
	 */
	public static function rebuildHtaccessFile() {
		self::rebuildConfigFile();
	}

	/**
	 * Writes rewrite rules to .htaccess file.
	 */
	public static function rebuildConfigFile() {
		// get options
		require_once(WCF_DIR.'lib/acp/option/Options.class.php');
		$options = Options::getOptionValues();
		
		if ($options['SEO_ENABLE']) {
			// open file
			if ($options['SEO_WEBSERVER'] == 'apache') {
				require_once(WCF_DIR.'lib/data/page/seo/ApacheRewriteRulesFile.class.php');
				$file = new ApacheRewriteRulesFile(WBB_DIR.'.htaccess');
				self::addRewriteRules($options, $file);
				$file->close();
				return;
			}
			else if ($options['SEO_WEBSERVER'] == 'lighttpd') {
				require_once(WCF_DIR.'lib/data/page/seo/LighttpdRewriteRulesFile.class.php');
				$file = new LighttpdRewriteRulesFile(WBB_DIR.'lighttpd.conf');
				self::addRewriteRules($options, $file);
				$file->close();
				return;
			}
		}
		
		// remove .htaccess file
		require_once(WCF_DIR.'lib/data/page/seo/SEOUtil.class.php');
		SEOUtil::removeSEO(WBB_DIR.'.htaccess');
	}
	
	protected static function addRewriteRules($options, RewriteRulesFile $file) {
		// write custom rules
		if ($options['SEO_CUSTOM_REWRITE_RULES']) {
			$file->addCustomRewriteRules($options['SEO_CUSTOM_REWRITE_RULES']);
		}
		
		// rewrite tag urls
		if ($options['SEO_REWRITE_TAG']) {
			$file->addRewriteRule($options['SEO_REWRITE_TAG_FORMAT'], 'index.php?page=TaggedObjects&tagID={1}', array('TAG_ID' => '[0-9]+', 'TAG_NAME' => '[^/\.]*'));
		}
		
		// rewrite user urls
		if ($options['SEO_REWRITE_USER_PROFILE']) {
			$file->addRewriteRule($options['SEO_REWRITE_USER_PROFILE_FORMAT'], 'index.php?page=User&userID={1}&username={2}', array('USER_ID' => '[0-9]+', 'USERNAME' => '[^/\.]*'));
		}
		
		// get parent boards pattern
		$parentBoardFormat = preg_quote($options['SEO_REWRITE_PARENT_BOARD_FORMAT']);
		$parentBoardFormat = str_replace('\{BOARD_ID\}', '[0-9]+', $parentBoardFormat);
		$parentBoardFormat = str_replace('\{BOARD_TITLE\}', '[^/\.]+', $parentBoardFormat);
		$parentBoardFormat = '('.$parentBoardFormat.')*';
		
		// rewrite thread urls
		if ($options['SEO_REWRITE_THREAD']) {
			$file->addRewriteRule($options['SEO_REWRITE_THREAD_LAST_POST_FORMAT'], 'index.php?page=Thread&threadID={3}&action=lastPost', array('BOARD_TITLE' => '[^/\.]+', 'THREAD_TOPIC' => '[^/\.]*', 'THREAD_ID' => '[0-9]+', 'BOARD_ID' => '[0-9]+', 'PARENT_BOARDS' => $parentBoardFormat));
			$file->addRewriteRule($options['SEO_REWRITE_THREAD_NEWEST_POST_FORMAT'], 'index.php?page=Thread&threadID={3}&action=firstNew', array('BOARD_TITLE' => '[^/\.]+', 'THREAD_TOPIC' => '[^/\.]*', 'THREAD_ID' => '[0-9]+', 'BOARD_ID' => '[0-9]+', 'PARENT_BOARDS' => $parentBoardFormat));
			$file->addRewriteRule($options['SEO_REWRITE_THREAD_POST_FORMAT'], 'index.php?page=Thread&postID={3}', array('BOARD_TITLE' => '[^/\.]+', 'THREAD_TOPIC' => '[^/\.]*', 'POST_ID' => '[0-9]+', 'THREAD_ID' => '[0-9]+', 'BOARD_ID' => '[0-9]+', 'PARENT_BOARDS' => $parentBoardFormat));
			$file->addRewriteRule($options['SEO_REWRITE_THREAD_MULTIPLE_FORMAT'], 'index.php?page=Thread&threadID={3}&pageNo={4}', array('BOARD_TITLE' => '[^/\.]+', 'THREAD_TOPIC' => '[^/\.]*', 'THREAD_ID' => '[0-9]+', 'THREAD_PAGE_NO' => '[0-9]+', 'BOARD_ID' => '[0-9]+', 'PARENT_BOARDS' => $parentBoardFormat));
			$file->addRewriteRule($options['SEO_REWRITE_THREAD_FORMAT'], 'index.php?page=Thread&threadID={3}', array('BOARD_TITLE' => '[^/\.]+', 'THREAD_TOPIC' => '[^/\.]*', 'THREAD_ID' => '[0-9]+', 'BOARD_ID' => '[0-9]+', 'PARENT_BOARDS' => $parentBoardFormat));
		}
		
		// rewrite board urls
		if ($options['SEO_REWRITE_BOARD']) {
			if (strpos($options['SEO_REWRITE_BOARD_MULTIPLE_FORMAT'], '{BOARD_ID}') !== false) {
				$file->addRewriteRule($options['SEO_REWRITE_BOARD_MULTIPLE_FORMAT'], 'index.php?page=Board&boardID={2}&pageNo={3}', array('BOARD_TITLE' => '[^/\.]+', 'BOARD_ID' => '[0-9]+', 'BOARD_PAGE_NO' => '[0-9]+', 'PARENT_BOARDS' => $parentBoardFormat));
			}
			else {
				$file->addRewriteRule($options['SEO_REWRITE_BOARD_MULTIPLE_FORMAT'], 'index.php?page=Board&boardTitle={1}&pageNo={2}', array('BOARD_TITLE' => '[^/\.]+', 'BOARD_PAGE_NO' => '[0-9]+', 'PARENT_BOARDS' => $parentBoardFormat));
			}
				
			if (strpos($options['SEO_REWRITE_BOARD_FORMAT'], '{BOARD_ID}') !== false) {
				$file->addRewriteRule($options['SEO_REWRITE_BOARD_FORMAT'], 'index.php?page=Board&boardID={2}', array('BOARD_TITLE' => '[^/\.]+', 'BOARD_ID' => '[0-9]+', 'PARENT_BOARDS' => $parentBoardFormat));
			}
			else {
				$file->addRewriteRule($options['SEO_REWRITE_BOARD_FORMAT'], 'index.php?page=Board&boardTitle={1}', array('BOARD_TITLE' => '[^/\.]+', 'PARENT_BOARDS' => $parentBoardFormat));
			}
		}
		
		// call event
		self::$options = $options;
		self::$file = $file;
		EventHandler::fireAction('WBBSEOUtil', 'addRewriteRules');
		
		// rewrite index url
		if ($options['SEO_REWRITE_INDEX']) {
			$file->addRewriteRule($options['SEO_REWRITE_INDEX_FORMAT'], 'index.php?page=Index', array('PAGE_TITLE' => '[^/\.]+'));
		}
	}
}
?>