<?php
// wcf imports
require_once(WBB_DIR.'lib/system/event/listener/URLParserThreadURLListener.class.php');

/**
 * Parses SEO URLs in messages.
 * 
 * @author	Marcel Werk
 * @copyright	2001-2009 WoltLab GmbH
 * @license	WoltLab Burning Board License <http://www.woltlab.com/products/burning_board/license.php>
 * @package	com.woltlab.wbb.seo
 * @subpackage	system.event.listener
 * @category 	Burning Board
 */
class SEOURLParserListener extends URLParserThreadURLListener {
	protected $threadURLPattern = null;
	protected $postURLPattern = null;
	
	/**
	 * @see EventListener::execute()
	 */
	public function execute($eventObj, $className, $eventName) {
		if (SEO_ENABLE && SEO_REWRITE_THREAD) {
			if ($this->threadURLPattern === null) {
				// get parent boards pattern
				$parentBoardFormat = preg_quote(SEO_REWRITE_PARENT_BOARD_FORMAT, '~');
				$parentBoardFormat = str_replace('\{BOARD_ID\}', '[0-9]+', $parentBoardFormat);
				$parentBoardFormat = str_replace('\{BOARD_TITLE\}', '[^/\.\s]+?', $parentBoardFormat);
				$parentBoardFormat = '(?:'.$parentBoardFormat.')*?';

				// thread urls
				$format = preg_quote(SEO_REWRITE_THREAD_FORMAT, '~');
				$keywords = array('BOARD_TITLE' => '[^\s]*?', 'BOARD_ID' => '\d+', 'THREAD_TOPIC' => '[^\s]*?', 'THREAD_ID' => '\d+', 'PARENT_BOARDS' => $parentBoardFormat);
				foreach ($keywords as $keyword => $pattern) {
					$format = StringUtil::replace('\{'.$keyword.'\}', ($keyword == 'THREAD_ID' ? '('.$pattern.')' : '(?:'.$pattern.')'), $format);
				}
				$this->threadURLPattern = '(?!/?user/|/?tag/)'.$format;
				
				// post urls
				$format = preg_quote(SEO_REWRITE_THREAD_POST_FORMAT, '~');
				$keywords = array('BOARD_TITLE' => '[^\s]*?', 'BOARD_ID' => '\d+', 'THREAD_TOPIC' => '[^\s]*?', 'POST_ID' => '\d+', 'PARENT_BOARDS' => $parentBoardFormat);
				foreach ($keywords as $keyword => $pattern) {
					$format = StringUtil::replace('\{'.$keyword.'\}', ($keyword == 'POST_ID' ? '('.$pattern.')' : '(?:'.$pattern.')'), $format);
				}
				$this->postURLPattern = '(?!/?user/|/?tag/)'.$format;
			}
			
			parent::execute($eventObj, $className, $eventName);
		}
	}
}
?>